@@ -1,6 +1,6 @@
 package main
 
-import "./extern_c"
+import "swigtests/extern_c"
 
 func main() {
 	extern_c.RealFunction(2)
