@@ -9,6 +9,6 @@ o = wostringstream();
 o << a << u" " << 2345 << u" " << 1.435 << wends;
 
 if (o.str() !=  "A class 2345 1.435\0")
-  error
+  error("failed");
 endif
 
