@@ -1,16 +1,17 @@
 <?php
 
 require "tests.php";
-require "director_detect.php";
 
 // No new functions
-check::functions(array('foo_cloner','foo_get_value','foo_get_class','foo_just_do_it','bar_baseclass','bar_cloner','bar_get_value','bar_get_class','bar_just_do_it'));
-// No new classes
+check::functions(array());
+// New classes
 check::classes(array('A','Foo','Bar'));
-// now new vars
+// No new vars
 check::globals(array());
 
 class MyBar extends Bar {
+  public $val;
+
   function __construct($val = 2) {
     parent::__construct();
     $this->val = $val;
@@ -52,4 +53,3 @@ check::equal($b->val, 5, "b: Bad virtual detection");
 check::equal($vc, 6, "c: Bad virtual detection");
 
 check::done();
-?>
