@@ -4,24 +4,38 @@
  * terms also apply to certain portions of SWIG. The full details of the SWIG
  * license and copyrights can be found in the LICENSE and COPYRIGHT files
  * included with the SWIG source code as distributed by the SWIG developers
- * and at http://www.swig.org/legal.html.
+ * and at https://www.swig.org/legal.html.
  *
  * expr.c
  *
  * Integer arithmetic expression evaluator used to handle expressions
  * encountered during preprocessing.
+ *
+ * Note that this is used for expressions in `#if` and the like, but not
+ * for expressions in `#define` which SWIG wraps as constants - for those
+ * we inject a `%constant` directive which is handled by the parser in
+ * `Source/CParse/parser.y`.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_expr_c[] = "$Id: expr.c 13939 2012-11-26 20:30:37Z wsfulton $";
-
 #include "swig.h"
 #include "preprocessor.h"
 
 static Scanner *scan = 0;
 
 typedef struct {
+  /* One of the EXPR_xxx values defined below. */
   int op;
+  /* op == EXPR_OP: value is the token specifying which operator.
+   *
+   * op == EXPR_VALUE && svalue == NULL: Numeric expression value.
+   *
+   * Otherwise unused.
+   */
   long value;
+  /* op == EXPR_VALUE: If non-NULL, string expression value; if NULL see value.
+   *
+   * Otherwise unused.
+   */
   String *svalue;
 } exprval;
 
@@ -29,7 +43,12 @@ typedef struct {
 #define  EXPR_VALUE    2
 #define  EXPR_OP       3
 #define  EXPR_GROUP    4
-#define  EXPR_UMINUS   100
+
+/* Special token values used here to distinguish from SWIG_TOKEN_MINUS
+ * and SWIG_TOKEN_PLUS (which we use here for a two argument versions).
+ */
+#define  OP_UMINUS   100
+#define  OP_UPLUS    101
 
 static exprval stack[256];	/* Parsing stack       */
 static int sp = 0;		/* Stack pointer       */
@@ -38,9 +57,11 @@ static int expr_init = 0;	/* Initialization flag */
 static const char *errmsg = 0;	/* Parsing error       */
 
 /* Initialize the precedence table for various operators.  Low values have higher precedence */
-static void init_precedence() {
+static void init_precedence(void) {
   prec[SWIG_TOKEN_NOT] = 10;
-  prec[EXPR_UMINUS] = 10;
+  prec[SWIG_TOKEN_LNOT] = 10;
+  prec[OP_UMINUS] = 10;
+  prec[OP_UPLUS] = 10;
   prec[SWIG_TOKEN_STAR] = 20;
   prec[SWIG_TOKEN_SLASH] = 20;
   prec[SWIG_TOKEN_PERCENT] = 20;
@@ -48,16 +69,15 @@ static void init_precedence() {
   prec[SWIG_TOKEN_MINUS] = 30;
   prec[SWIG_TOKEN_LSHIFT] = 40;
   prec[SWIG_TOKEN_RSHIFT] = 40;
-  prec[SWIG_TOKEN_AND] = 50;
-  prec[SWIG_TOKEN_XOR] = 60;
-  prec[SWIG_TOKEN_OR] = 70;
-  prec[SWIG_TOKEN_EQUALTO] = 80;
-  prec[SWIG_TOKEN_NOTEQUAL] = 80;
-  prec[SWIG_TOKEN_LESSTHAN] = 80;
-  prec[SWIG_TOKEN_GREATERTHAN] = 80;
-  prec[SWIG_TOKEN_LTEQUAL] = 80;
-  prec[SWIG_TOKEN_GTEQUAL] = 80;
-  prec[SWIG_TOKEN_LNOT] = 90;
+  prec[SWIG_TOKEN_LESSTHAN] = 50;
+  prec[SWIG_TOKEN_GREATERTHAN] = 50;
+  prec[SWIG_TOKEN_LTEQUAL] = 50;
+  prec[SWIG_TOKEN_GTEQUAL] = 50;
+  prec[SWIG_TOKEN_EQUALTO] = 60;
+  prec[SWIG_TOKEN_NOTEQUAL] = 60;
+  prec[SWIG_TOKEN_AND] = 70;
+  prec[SWIG_TOKEN_XOR] = 80;
+  prec[SWIG_TOKEN_OR] = 90;
   prec[SWIG_TOKEN_LAND] = 100;
   prec[SWIG_TOKEN_LOR] = 110;
   expr_init = 1;
@@ -65,11 +85,12 @@ static void init_precedence() {
 
 #define UNARY_OP(token) (((token) == SWIG_TOKEN_NOT) || \
 			 ((token) == SWIG_TOKEN_LNOT) || \
-			 ((token) == EXPR_UMINUS))
+			 ((token) == OP_UMINUS) || \
+			 ((token) == OP_UPLUS))
 
 /* Reduce a single operator on the stack */
 /* return 0 on failure, 1 on success */
-static int reduce_op() {
+static int reduce_op(void) {
   long op_token = stack[sp - 1].value;
   assert(sp > 0);
   assert(stack[sp - 1].op == EXPR_OP);
@@ -185,17 +206,31 @@ static int reduce_op() {
       stack[sp - 1].value = !stack[sp].value;
       sp--;
       break;
-    case EXPR_UMINUS:
+    case OP_UMINUS:
       stack[sp - 1].value = -stack[sp].value;
       sp--;
       break;
+    case OP_UPLUS:
+      stack[sp - 1].value = stack[sp].value;
+      sp--;
+      break;
     case SWIG_TOKEN_SLASH:
-      stack[sp - 2].value = stack[sp - 2].value / stack[sp].value;
-      sp -= 2;
+      if (stack[sp].value != 0) {
+	stack[sp - 2].value = stack[sp - 2].value / stack[sp].value;
+	sp -= 2;
+      } else {
+	errmsg = "Division by zero in expression";
+	return 0;
+      }
       break;
     case SWIG_TOKEN_PERCENT:
-      stack[sp - 2].value = stack[sp - 2].value % stack[sp].value;
-      sp -= 2;
+      if (stack[sp].value != 0) {
+	stack[sp - 2].value = stack[sp - 2].value % stack[sp].value;
+	sp -= 2;
+      } else {
+	errmsg = "Modulo by zero in expression";
+	return 0;
+      }
       break;
     case SWIG_TOKEN_LSHIFT:
       stack[sp - 2].value = stack[sp - 2].value << stack[sp].value;
@@ -270,13 +305,15 @@ int Preprocessor_expr(DOH *s, int *error) {
 
   /* Put initial state onto the stack */
   stack[sp].op = EXPR_TOP;
-  stack[sp].value = 0;
 
   while (1) {
     /* Look at the top of the stack */
     switch (stack[sp].op) {
     case EXPR_TOP:
-      /* An expression.   Can be a number or another expression enclosed in parens */
+      /* EXPR_TOP is a place-holder which can only appear on the top of the
+       * stack.  We can reduce it to any expression - a number, a string, an
+       * unary operator, or another expression enclosed in parentheses.
+       */
       token = expr_token(scan);
       if (!token) {
 	errmsg = "Expected an expression";
@@ -286,36 +323,49 @@ int Preprocessor_expr(DOH *s, int *error) {
       if ((token == SWIG_TOKEN_INT) || (token == SWIG_TOKEN_UINT) || (token == SWIG_TOKEN_LONG) || (token == SWIG_TOKEN_ULONG)) {
 	/* A number.  Reduce EXPR_TOP to an EXPR_VALUE */
 	char *c = Char(Scanner_text(scan));
-	stack[sp].value = (long) strtol(c, 0, 0);
+	if (c[0] == '0' && (c[1] == 'b' || c[1] == 'B')) {
+	  /* strtol() doesn't handle binary constants */
+	  stack[sp].value = (long) strtol(c + 2, 0, 2);
+	} else {
+	  stack[sp].value = (long) strtol(c, 0, 0);
+	}
 	stack[sp].svalue = 0;
-	/*        stack[sp].value = (long) atol(Char(Scanner_text(scan))); */
 	stack[sp].op = EXPR_VALUE;
-      } else if (token == SWIG_TOKEN_PLUS) {
-      } else if ((token == SWIG_TOKEN_MINUS) || (token == SWIG_TOKEN_LNOT) || (token == SWIG_TOKEN_NOT)) {
+      } else if ((token == SWIG_TOKEN_MINUS) || (token == SWIG_TOKEN_PLUS) || (token == SWIG_TOKEN_LNOT) || (token == SWIG_TOKEN_NOT)) {
 	if (token == SWIG_TOKEN_MINUS)
-	  token = EXPR_UMINUS;
+	  token = OP_UMINUS;
+	else if (token == SWIG_TOKEN_PLUS)
+	  token = OP_UPLUS;
 	stack[sp].value = token;
-	stack[sp++].op = EXPR_OP;
+	stack[sp].op = EXPR_OP;
+	sp++;
 	stack[sp].op = EXPR_TOP;
-	stack[sp].svalue = 0;
       } else if (token == SWIG_TOKEN_LPAREN) {
-	stack[sp++].op = EXPR_GROUP;
+	stack[sp].op = EXPR_GROUP;
+	sp++;
 	stack[sp].op = EXPR_TOP;
-	stack[sp].value = 0;
-	stack[sp].svalue = 0;
       } else if (token == SWIG_TOKEN_ENDLINE) {
       } else if (token == SWIG_TOKEN_STRING) {
 	stack[sp].svalue = NewString(Scanner_text(scan));
 	stack[sp].op = EXPR_VALUE;
       } else if (token == SWIG_TOKEN_ID) {
+	/* Defined macros have been expanded already so this is an unknown
+	 * macro, which gets treated as zero.
+	 */
 	stack[sp].value = 0;
 	stack[sp].svalue = 0;
 	stack[sp].op = EXPR_VALUE;
+      } else if ((token == SWIG_TOKEN_FLOAT) || (token == SWIG_TOKEN_DOUBLE)) {
+	errmsg = "Floating point constant in preprocessor expression";
+	*error = 1;
+	return 0;
       } else
 	goto syntax_error;
       break;
     case EXPR_VALUE:
-      /* A value is on the stack.   We may reduce or evaluate depending on what the next token is */
+      /* A value is on top of the stack.  We may reduce or evaluate depending
+       * on what the next token is.
+       */
       token = expr_token(scan);
       if (!token) {
 	/* End of input. Might have to reduce if an operator is on stack */
@@ -359,7 +409,6 @@ int Preprocessor_expr(DOH *s, int *error) {
 	  stack[sp].value = token;
 	  sp++;
 	  stack[sp].op = EXPR_TOP;
-	  stack[sp].value = 0;
 	} else {
 	  if (stack[sp - 1].op != EXPR_OP)
 	    goto syntax_error_expected_operator;
@@ -378,7 +427,6 @@ int Preprocessor_expr(DOH *s, int *error) {
 	  stack[sp].value = token;
 	  sp++;
 	  stack[sp].op = EXPR_TOP;
-	  stack[sp].value = 0;
 	}
 	break;
       case SWIG_TOKEN_RPAREN:
@@ -394,8 +442,11 @@ int Preprocessor_expr(DOH *s, int *error) {
 	  goto extra_rparen;
 	stack[sp - 1].op = EXPR_VALUE;
 	stack[sp - 1].value = stack[sp].value;
+	stack[sp - 1].svalue = stack[sp].svalue;
 	sp--;
 	break;
+      case SWIG_TOKEN_LTEQUALGT:
+	goto spaceship_not_allowed;
       default:
 	goto syntax_error_expected_operator;
 	break;
@@ -404,7 +455,7 @@ int Preprocessor_expr(DOH *s, int *error) {
 
     default:
       fprintf(stderr, "Internal error in expression evaluator.\n");
-      abort();
+      Exit(EXIT_FAILURE);
     }
   }
 
@@ -427,6 +478,11 @@ extra_rparen:
   errmsg = "Extra \')\'";
   *error = 1;
   return 0;
+
+spaceship_not_allowed:
+  errmsg = "Spaceship operator (<=>) not allowed in preprocessor expression";
+  *error = 1;
+  return 0;
 }
 
 /* -----------------------------------------------------------------------------
@@ -435,6 +491,6 @@ extra_rparen:
  * Return error message set by the evaluator (if any)
  * ----------------------------------------------------------------------------- */
 
-const char *Preprocessor_expr_error() {
+const char *Preprocessor_expr_error(void) {
   return errmsg;
 }
